@@ -108,7 +108,7 @@ class TestCMS(NereidTestCase):
             'application_user': USER,
             'default_locale': self.locale_en_us.id,
             'guest_user': guest_user,
-            'currencies': [('set', [usd.id])],
+            'currencies': [('add', [usd.id])],
         }])
 
         # Create an article category
@@ -276,6 +276,18 @@ class TestCMS(NereidTestCase):
             self.Article.delete([article1])
             self.assertEqual(self.ArticleAttribute.search([], count=True), 0)
 
+    def test_0060_create_menu_item(self):
+        '''
+        Test creating a new menu item
+        '''
+        with Transaction().start(DB_NAME, USER, CONTEXT):
+            menuitem, = self.MenuItem.create([{
+                'title': 'MenuItem',
+                'unique_name': 'MenuItem',
+                'sequence': 1
+            }])
+            self.assert_(menuitem)
+
 
 def suite():
     "CMS test suite"
